import 'package:mongo_dart/mongo_dart.dart' as mgo;
import 'package:jaguar_serializer/serializer.dart';
import 'package:jaguar_data_store/jaguar_data_store.dart';

class Post implements Idied<String> {
  String id;

  String title;

  String message;

  int likes;

  Post();

  Post.build(this.id, this.title, this.message, this.likes);

  Post.buildNoId(this.title, this.message, this.likes);

  String toString() =>
      "Post(id: $id, title: $title, message: $message, likes: $likes)";

  bool operator==(object) {
    if(object is! Post) return false;

    final Post post = object as Post;

    if(id != post.id) return false;
    if(title != post.title) return false;
    if(message != post.message) return false;
    if(likes != post.likes) return false;

    return true;
  }
}

class PostSerializer extends MapSerializer<Post> {
  PostSerializer();

  Map toMap(Post model) => {
    "_id": model.id,
    "title": model.title,
    "message": model.message,
    "likes": model.likes,
  };

  Post fromMap(Map map, {Post model}) {
    final ret = super.fromMap(map);

    if(map is! Map) return null;

    {
      dynamic value = map["_id"];
      if (value is String) {
        ret.id = value;
      }
    }

    {
      dynamic value = map["title"];
      if (value is String) {
        ret.title = value;
      }
    }

    {
      dynamic value = map["message"];
      if (value is String) {
        ret.message = value;
      }
    }

    {
      dynamic value = map["likes"];
      if (value is int) {
        ret.likes = value;
      }
    }

    return ret;
  }

  Post createModel() => new Post();
}

class MongoPostSerializer extends MapSerializer<Post> {
  MongoPostSerializer();

  Map toMap(Post model) => {
    "_id": mgo.ObjectId.parse(model.id),
    "title": model.title,
    "message": model.message,
    "likes": model.likes,
  };

  Post fromMap(Map map, {Post model}) {
    final ret = super.fromMap(map);

    if(map is! Map) return null;

    {
      dynamic value = map["_id"];
      if (value is mgo.ObjectId) {
        ret.id = value.toHexString();
      }
    }

    {
      dynamic value = map["title"];
      if (value is String) {
        ret.title = value;
      }
    }

    {
      dynamic value = map["message"];
      if (value is String) {
        ret.message = value;
      }
    }

    {
      dynamic value = map["likes"];
      if (value is int) {
        ret.likes = value;
      }
    }

    return ret;
  }

  Post createModel() => new Post();
}

final PostSerializer serializer = new PostSerializer();

final MongoPostSerializer mongoSerializer = new MongoPostSerializer();