library mongo_data_store.resource;

import 'dart:async';
import 'package:jaguar/jaguar.dart';
import 'package:jaguar/interceptors.dart';
import 'package:jaguar_mongo/jaguar_mongo.dart';
import 'package:mongo_dart/mongo_dart.dart' as mgo;
import 'package:jaguar_serializer/serializer.dart';
import 'package:jaguar_data_store/jaguar_data_store.dart';
import 'mongo_data_store.dart';

@RouteGroup()
@Wrap(const [#mongoWrapper])
@WrapEncodeToJson()
class MongoResource<ModelType extends Idied<String>> {
  final String mongoUrl;

  final String collectionName;

  final MapSerializer<ModelType> jsonSerializer;

  final MapSerializer<ModelType> mongoSerializer;

  MongoDataStore<ModelType> createStore(mgo.Db db) =>
      new MongoDataStore<ModelType>(mongoSerializer, collectionName, db);

  MongoResource(this.mongoUrl, this.collectionName, this.jsonSerializer,
      this.mongoSerializer);

  @Get()
  Future<List<Map>> getAll(@Input(MongoDb) mgo.Db db) => createStore(db)
      .getAll()
      .map((ModelType m) => jsonSerializer.toMap(m))
      .toList();

  @Get(path: '/:id')
  Future<Map> getById(@Input(MongoDb) mgo.Db db, String id) async {
    final ModelType m = await createStore(db).getById(id);
    return jsonSerializer.toMap(m);
  }

  @Post()
  @WrapDecodeJsonMap()
  Future<String> create(
      @Input(MongoDb) mgo.Db db, @Input(DecodeJsonMap) Map data) async {
    final ModelType m = jsonSerializer.fromMap(data);
    //TODO validate
    return await createStore(db).insert(m);
  }

  @Put()
  @WrapDecodeJsonMap()
  Future<int> update(
      @Input(MongoDb) mgo.Db db, @Input(DecodeJsonMap) Map data) async {
    final ModelType m = jsonSerializer.fromMap(data);
    //TODO validate
    return await createStore(db).updateById(m.id, m);
  }

  @Delete(path: '/:id')
  Future<int> deleteById(@Input(MongoDb) mgo.Db db, String id) =>
      createStore(db).deleteById(id);

  @Delete()
  Future<int> deleteAll(@Input(MongoDb) mgo.Db db) => createStore(db).deleteAll();

  WrapMongoDb mongoWrapper() => new WrapMongoDb(mongoUrl);
}
